/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui.tray;

import gui.*;
import controller.*;
import agent.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.*;
import java.io.*;
import java.util.*;




/* ====================================================================== */
/** This class suplies a few utilities for the user to perform on the
 *  simulatio such as finding agents, changes maps, toggling wireless
 *  ranges, etc.
 *
 */
public class PnlTabTools extends JPanel implements ControllerListener {

	/** The simulation controller that keeps all data. */
	private Controller controller;

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

	//Components in this JPanel

	/** Main panel to hold all components */
    private JPanel pnlMain = new JPanel();

	/** Combo Box for finding an agent. */
    private JComboBox cmbFind = new JComboBox();

	/** Combo Box for changing the map. */
    private JComboBox cmbChangeMap = new JComboBox();

	/** Button toggles the visiblity of wireless ranges. */
    private JButton btnShowRanges = new JButton("Show wireless ranges");

	/** Button resets all data, deletes all agents. */
    private JButton btnRefresh = new JButton("Remove all agents from map");



    // ======================================================================
    /** Constructor sets up this Panel for a tab and calls init.
     *  It also signs us up as a controller listener.
     *
     */
    public PnlTabTools( Controller controller ) {

		this.controller = controller;
		controller.addControllerListener(this);

        try {
            init();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }

    } //Constructor




    // ======================================================================
    /** Initializes this JPanel with Components.  Adds comboboxes, buttons,
     *  and sets the action listeners.
     *
     */
    private void init() throws Exception {

		// Setup layouts and borders
        bLayMain.setHgap(5);
        bLayMain.setVgap(5);
        this.setBorder(TTBorder.getBorder("Tools"));
        this.setLayout(bLayMain);
        pnlMain.setLayout(gbagLayInfo);

		// Setup the comboboxes
        cmbFind.addItem( "Find Agent... " );
        cmbChangeMap.addItem("Change Map...");
        scanDirectory();


		// Add stuff to panels
        this.add(pnlMain, BorderLayout.WEST);

        pnlMain.add(cmbFind,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        pnlMain.add(cmbChangeMap,              new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));


        pnlMain.add(btnShowRanges,              new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        pnlMain.add(btnRefresh,              new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

		// Setup the action listeners
        cmbFind.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  cmbFind_actionPerformed();
              }
        });

        cmbChangeMap.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  cmbChangeMap_actionPerformed();
              }
        });

        btnShowRanges.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btnShowRanges_actionPerformed();
              }
        });

        btnRefresh.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btnReset_actionPerformed();
              }
        });

		// Set panel size (gets over ridden by TrayDialog's set size)
        this.setSize(new Dimension(550, 300));


    } //method: init




    // ========================================================================
    /** This method is called to find an agent on the simulation map.
     *
     */
    public void cmbFind_actionPerformed() {

		//Get the selection in the combo box
		String choice = (String)(cmbFind.getSelectedItem());

		Agent a = controller.centerAgent(choice);

    } //method: cmbFind_actionPerformed



    // ========================================================================
    /** This method is called to change the map on the simulation frame.
     *
     */
    public void cmbChangeMap_actionPerformed() {

		String mapname = (String)cmbChangeMap.getSelectedItem();

		if( mapname.indexOf("Change Map") < 0 ){
			controller.changeChart(mapname);

		}

	} //method: cmbChangeMap_actionPerformed



    // ========================================================================
    /** This method is called to show/hide wireless ranges on the map.
     *
     */
    public void btnShowRanges_actionPerformed() {

		String st = btnShowRanges.getText();

		if(st.indexOf("Show") < 0 ){
			btnShowRanges.setText("Show wireless ranges");
			controller.setRanges(false);
		}else{
			btnShowRanges.setText(" Hide wireless ranges ");
			controller.setRanges(true);
		}

    } //method: btnShowRanges_actionPerformed



    // ========================================================================
    /** This method is called to reset the simulation data.
     *
     */
    public void btnReset_actionPerformed() {

		controller.reset();

    } //method: btnReset_actionPerformed



    // ========================================================================
    /** This method required by ControllerListener interface.
     *  This method resets the data in the Find Agent combobox.
     *
     */
    public void refresh(){

		Vector users = controller.getUsers();
		Vector points = controller.getPoints();
		Agent use;

		cmbFind.removeAllItems();
		cmbFind.addItem( "Find... " );

		// Add users
		for (int i=0; i < users.size(); i++) {
			use = (Agent)users.get(i);
			cmbFind.addItem(use.getName() );
		}

		// Add access points
		for (int i=0; i < points.size(); i++) {
			use = (Agent)points.get(i);
			cmbFind.addItem(use.getName() );
		}

	} //method: refresh




   	/* ====================================================================== */
	/** This method is a private method for this class only.  It provides a
	 *  way to scan the xml directory inside Simulator which contains
	 *  all the xml templates for maps.
	 *
	 */
	private void scanDirectory(){

     	try{

			// create any file in current directory
			File currentDir = new File("x");
			// get the full path
			currentDir = new File(currentDir.getAbsolutePath());
			// and lose the fake filename.
    		currentDir = new File(currentDir.getParent());
			//now go to our subdir
    		currentDir = new File(currentDir, "files/maps");

			String[] files = currentDir.list();

     		for (int j=0; j < files.length; j++){

     			int index = files[j].indexOf(".map");

     			if (index != -1){
     				cmbChangeMap.addItem(files[j]);
				}
	    	}

     	}catch(Exception e){}

  	} //method: scanDirectory


} //class: PnlTabTools